@@ -43,6 +43,7 @@ FILE *log_file = NULL;
 glong scroll_previous_run;
 GtkStatusIcon *trayIcon = NULL;
 GtkTreePath *path_set = NULL;
+GtkTreeModelFilter *filter = NULL;
 gboolean more = FALSE, first = TRUE;
 
 
@@ -406,7 +407,6 @@ gboolean load_groups(gchar *session) {
 		GKeyFile *settings_file;
 		GtkComboBox *combo;
 		GtkTreeIter iter;
-		static GtkTreeModelFilter *filter = NULL;
 		gint i;
 		gboolean flag = FALSE;
 
@@ -830,10 +830,26 @@ gboolean on_main_configure_event(GtkWidget *widget, GdkEventConfigure *event, gp
 }
 
 
+void superkill(int sig) {
+	if (get_checkbox("check_superuser")) {
+		gchar *cmd;
+		gchar pidbuf[6], sigbuf[6];
+
+		sprintf(pidbuf, "%i", rsync_pid);
+		sprintf(sigbuf, "%i", sig);
+		cmd = g_strconcat("pkexec kill -", sigbuf, " ", pidbuf, NULL);
+		g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
+		g_free(cmd);
+	} else {
+		kill(rsync_pid, sig);
+	}
+}
+
+
 void on_close_clicked(GtkButton *button, gpointer user_data) {
 	if (rsync_pid) {
-		if (paused) kill(rsync_pid, SIGCONT);
-		kill(rsync_pid, SIGTERM);
+		if (paused) superkill(SIGCONT);
+		superkill(SIGTERM);
 		gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), FALSE);
 	} else {
 		if (more) on_play_clicked(NULL, NULL);		//on_rsync_show(NULL, NULL);
@@ -849,7 +865,8 @@ void on_close_clicked(GtkButton *button, gpointer user_data) {
 void scroll_to_end(GtkTextView* view, gboolean final) {
 	GTimeVal curtime;
 
-	if (config_fastscroll && !final) g_get_current_time(&curtime);
+//	if (config_fastscroll && !final) g_get_current_time(&curtime);
+	g_get_current_time(&curtime);
 	if (!config_fastscroll || final || curtime.tv_sec - scroll_previous_run >= 1) {
 		GtkTextBuffer *buffer;
 		GtkTextIter iter;
@@ -1159,8 +1176,8 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 
 void on_rsync_hide(GtkObject *object, gpointer user_data) {
 	if (rsync_pid) {
-		if (paused) kill(rsync_pid, SIGCONT);
-		kill(rsync_pid, SIGTERM);
+		if (paused) superkill(SIGCONT);
+		superkill(SIGTERM);
 	}
 	dryrunning = FALSE;
 }
@@ -1186,6 +1203,8 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 	result = gtk_dialog_run ((GtkDialog*) add);
 
 	if (result == GTK_RESPONSE_OK) {
+		save_settings(groups[session_last], NULL);
+
 		gchar *newses;
 		newses = (gchar*) gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_session_name"));
 		is_set = get_checkbox("checkbutton_isset");
@@ -1221,7 +1240,6 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 			return;
 		}
 
-		save_settings(groups[session_last], NULL);
 		save_settings(newses, NULL);
 		session_number++;
 		session_last = -1;		// added because of save session bug, copied from external patch
@@ -1340,14 +1358,14 @@ void on_button_pause_clicked(GtkButton *button, gpointer user_data) {
 	GtkButton *tmpbutt = (GtkButton*) gtk_builder_get_object(builder, "button_pause");
 	GTimeVal tmptime;
 	if (paused) {
-		if (rsync_pid) kill(rsync_pid, SIGCONT);
+		if (rsync_pid) superkill(SIGCONT);
 		gtk_button_set_label(tmpbutt, "gtk-media-pause");
 		set_rsync_title(groups[session_last], "running");
 		g_get_current_time(&tmptime);
 		startime.tv_sec += tmptime.tv_sec - pausedtime.tv_sec;
 		startime.tv_usec += tmptime.tv_usec - pausedtime.tv_usec;
 	} else {
-		if (rsync_pid) kill(rsync_pid, SIGSTOP);
+		if (rsync_pid) superkill(SIGSTOP);
 		gtk_button_set_label(tmpbutt, "gtk-media-play");
 		set_rsync_title(groups[session_last], "paused");
 		g_get_current_time(&pausedtime);
@@ -1571,12 +1589,14 @@ void on_contribute_activate(GtkMenuItem *menuitem, gpointer user_data) {
 
 void on_cellrenderertoggle1_toggled(GtkCellRendererToggle *cell, gchar *path_str, gpointer data) {
 	GtkTreeIter iter;
+	GtkTreePath *path_filter = gtk_tree_path_new_from_string(path_str);
+	GtkTreePath *path = gtk_tree_model_filter_convert_path_to_child_path(filter, path_filter);
 	gboolean enabled;
-	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
 
 	gtk_tree_model_get_iter((GtkTreeModel*) liststore_session, &iter, path);
-	gtk_tree_path_free(path);
 	gtk_tree_model_get((GtkTreeModel*) liststore_session, &iter, 1, &enabled, -1);
 	enabled = !enabled;
 	gtk_list_store_set(liststore_session, &iter, 1, enabled, -1);
+	gtk_tree_path_free(path_filter);
+	gtk_tree_path_free(path);
 }
